classdef BoundaryCondition
    
    
    properties
        type
        edges
        g
        h
    end
    
    methods
        
        function obj=BoundaryCondition(type,edges,g,h)
            % % BOUNDARYCONDITION : BoundaryCondition class.
            %
            %       This class allows an easy definition of a large class
            %       of boundary conditions ascociated to a pde problem.
            %
            %       Constructors :
            %
            %       obj=BoundaryCondition(edges) creates a
            %       BoundaryCondition object representing a 0 Dirichlet
            %       condition on the edges precised in the rows edges.
            %
            %       
            %       obj=BoundaryCondition(type,edges) creates a
            %       BoundaryCondition object, type = 1 for 0 Dirichlet and
            %       type = 2 for 0 Neumann condition on the edges precised
            %       in the indices row edges.
            %
            %       obj=BoundaryCondition(type,edges,g) creates a
            %       BoundaryCondition object. If type = 1 it is a Dirichlet
            %       condition u = g, if type = 2 it a Neumann condition
            %       cxgrad(u).n = g on the boundary part defined with the
            %       indices row edges.
            %       
            %       
            %       obj=BoundaryCondition(type,edges,g,h) creates a
            %       BoundaryCondition object. type = 2, it defines a
            %       boundary condition cxgrad(u).n + hu = g.
            %
            %       Remark : g and h can be given in a large kind of type.
            %       See assembpde for description.
            %
            %       Methods :
            %
            switch nargin
                case 1
                    obj.type=1;
                    obj.edges=type;
                    obj.g='0';
                    obj.h='';
                case 2
                    if type==1
                        obj.type=1;
                        obj.edges=edges;
                        obj.g='0';
                        obj.h='';
                    elseif type==2
                        obj.type=2;
                        obj.edges=edges;
                        obj.g='0';
                        obj.h='0';
                    end
                case 3
                    if type==1
                        obj.type=1;
                        obj.edges=edges;
                        obj.g=g;
                        obj.h='';
                    elseif type==2
                        obj.type=2;
                        obj.edges=edges;
                        obj.g=g;
                        obj.h='0';
                    end
                    case 4
                    if type==1
                        disp('Bad boundary condition definition!')
                    elseif type==2
                        obj.type=2;
                        obj.edges=edges;
                        obj.g=g;
                        obj.h=h;
                    end
            end
                        
                        
           
                    
            
        end
        
        function disp(obj)
            if obj.type==1
                disp('Dirichlet boundary condition u = g')
                disp(['g = ', obj.g])
                disp(['Boundary segments : ', num2str(obj.edges)])
            elseif obj.type==2
                disp('Neumann/Robin boundary condition n.grad(u)+hu = g')
                disp(['g = ', obj.g])
                disp(['h = ', obj.h])
                disp(['Boundary segments : ', num2str(obj.edges)])
            else
                disp('Void boundary condition!')
            end
        end
                 
        
    end
    
    
end
    