clear all
close all


%% Construction of phantom
% Rectangular domain
bound=Boundary([0 2 2 0;0 0 1 1]);
res=512;
ny=res;
nx=2*res;
x=linspace(0,2,2*res);
y=linspace(0,1,res);
[X,Y]=meshgrid(x,y);
p=zeros(res,2*res);
p(:,res/2:res/2+res-1)=phantom(res);
s_grid=1 + (X>1/2+1/20*sin(1+10*Y)).*(X<3/2+1/20*sin(10+15*Y)) + p;
s_grid=fliplr(s_grid')';
sigmareal=s_grid;
imagesc(x,y,sigmareal)
axis equal
axis xy
colorbar
title('Real conductivity')

% figure
% imagesc(x,y,sigmareal)

rafinement=0.1;
m=Mesh(bound,2);
%m.plot

%% iterative refining method for the mesh.
niter=3;
for i=1:niter
    %m.plot
    p=m.nodes;t=m.triangles;
    bc1=BoundaryCondition(1,[1,2,3,4],'y');BCM=boundaryconditionsmatrix(bc1);
    sigmarealnod=m.grid2mesh(X,Y,sigmareal); sigmarealtri=m.nod2tri(sigmarealnod);
    umes=m.solvepde(BCM,sigmarealtri,'0','0');
    gumestri=m.grad(umes);gumesnod=m.tri2nod(gumestri);mesurestri=[sigmarealtri.*gumestri(1,:);sigmarealtri.*gumestri(2,:)];
    normmesures=sqrt(mesurestri(1,:).^2 +mesurestri(2,:).^2);
    D= [1./sqrt((mesurestri(1,:).^2 +mesurestri(2,:).^2)).*mesurestri(1,:);1./sqrt((mesurestri(1,:).^2 +mesurestri(2,:).^2)).*mesurestri(2,:)];
    E=[-D(2,:);D(1,:)];
    epsilon=0.0001;
    C1=E(1,:).^2+epsilon;
    C2=E(1,:).*E(2,:);
    C3=E(2,:).^2+epsilon;
    C=[C1;C2;C3];
    U2=m.solvepde(BCM,C,0,0);
    gu2=m.grad(U2);normgu2=sqrt(gu2(1,:).^2+gu2(2,:).^2);
    sigmarecon=normmesures./normgu2;
    sigmareconnod=m.tri2nod(sigmarecon);
    m2=m.refine(bound,sigmareconnod,rafinement);
    if i<niter
        m=m2;
    end
end

%% Measurements construction

noise=0.0;
a=pdetrg(p,t);
aire=sum(a);
normmesures=sqrt(mesurestri(1,:).^2 +mesurestri(2,:).^2);
norml2mesures=sqrt(1/aire *sum(a.*normmesures.^2));
mesurestri2=[mesurestri(1,:)+noise*norml2mesures*randn(1,length(mesurestri(1,:)));mesurestri(2,:)+noise*norml2mesures*randn(1,length(mesurestri(2,:)))];

%% Optimal control algorithm

Sigma=2*ones(ny,nx) ;
%Sigma=sigmareal;

sigmanod=m.grid2mesh(X,Y,Sigma);
sigmatri=m.nod2tri(sigmanod);

dT=.1;
nmax=50;
JJ=zeros(1,nmax); % J operator values


for i=1:nmax
    % Compute F[sigma] and \nabla p, the dual problem solution
    [U0,gutri,p,dptri]=fonction(m,sigmatri,sigmanod,mesurestri2);
    % Calculate the Measurements error
    JJ(i)=fonctionnelleJ(m,sigmatri,mesurestri2,gutri);
    % Compute the gradient of J
    dJ=gradJ(mesurestri2,sigmatri,gutri,dptri);

    % Plot current conductivity value and derivative direction
    FIG=figure(2);
    subplot(1,2,1)
    m.surf(sigmatri)
    colorbar
    title(['Sigma iter ', num2str(i)])
    subplot(1,2,2)
    m.surf(dJ)
    colorbar
    title(['dJ iter ', num2str(i)])
    figureFullScreen(FIG)
    pause(1/16)
    % Gradient descent step.
    sigmatri=sigmatri-dT*dJ;
    
end
figure
 imagesc(x,y,m.mesh2grid(x,y,m.tri2nod(sigmatri)))
 axis xy
 axis equal
 axis([0 2 0 1]);
 colorbar
 title('Reconstruction')

figure
plot(log(JJ))
title('J[\sigma_i] in log-scale')
